@@ -94,8 +94,7 @@ def train_from_folder(
     dataset_aug_prob = 0.,
     multi_gpus = False,
     calculate_fid_every = None,
-    seed = 42,
-    activation = 'leaky_relu'
+    seed = 42
 ):
     model_args = dict(
         name = name,
@@ -115,8 +114,7 @@ def train_from_folder(
         aug_prob = aug_prob,
         aug_types = cast_list(aug_types),
         dataset_aug_prob = dataset_aug_prob,
-        calculate_fid_every = calculate_fid_every,
-        activation = activation
+        calculate_fid_every = calculate_fid_every
     )
 
     if generate:
