@@ -17,6 +17,7 @@ from torch.utils.data import Dataset, DataLoader
 from torch.autograd import grad as torch_grad
 from torch.utils.data.distributed import DistributedSampler
 from torch.nn.parallel import DistributedDataParallel as DDP
+from torch.nn.utils import weight_norm as WN
 
 from PIL import Image
 import torchvision
@@ -87,15 +88,6 @@ def gradient_accumulate_contexts(gradient_accumulate_every, is_ddp, ddps):
         with context():
             yield
 
-def gradient_penalty(images, outputs, weight = 10):
-    batch_size = images.shape[0]
-    gradients = torch_grad(outputs=outputs, inputs=images,
-                           grad_outputs=list(map(lambda t: torch.ones(t.size(), device = images.device), outputs)),
-                           create_graph=True, retain_graph=True, only_inputs=True)[0]
-
-    gradients = gradients.reshape(batch_size, -1)
-    return weight * ((gradients.norm(2, dim=1) - 1) ** 2).mean()
-
 def evaluate_in_chunks(max_batch_size, model, *args):
     split_args = list(zip(*list(map(lambda x: x.split(max_batch_size, dim=0), args))))
     chunked_outputs = [model(*i) for i in split_args]
@@ -111,86 +103,6 @@ def slerp(val, low, high):
     res = (torch.sin((1.0 - val) * omega) / so).unsqueeze(1) * low + (torch.sin(val * omega) / so).unsqueeze(1) * high
     return res
 
-# activation classes
-
-class Activation(nn.Module):
-    def __init__(self, fn):
-        super().__init__()
-        self.fn = fn
-    def forward(self, x):
-        return self.fn(x)
-
-class MishFn(torch.autograd.Function):
-    @staticmethod
-    def forward(ctx, x):
-        x_tanh_sp = torch.nn.functional.softplus(x).tanh()
-        if x.requires_grad:
-            ctx.save_for_backward(x_tanh_sp + x * x.sigmoid() * (1 - x_tanh_sp.square()))
-        y = x * x_tanh_sp
-        return y
-
-    @staticmethod
-    def backward(ctx, grad_output):
-        if len(ctx.saved_tensors) == 0:
-            return None
-        grad, = ctx.saved_tensors
-        return grad_output * grad
-    
-    
-class SwishFn(torch.autograd.Function):
-    @staticmethod
-    def forward(ctx, i):
-        with torch.no_grad():
-            sigmoid_i = i.sigmoid()
-            result = i * sigmoid_i
-            if i.requires_grad:
-                grad = sigmoid_i + result * (1 - sigmoid_i)
-                ctx.save_for_backward(grad)
-        result.requires_grad_(i.requires_grad)
-        return result
-
-    @staticmethod
-    def backward(ctx, grad_output):
-        grad, = ctx.saved_tensors
-        return grad_output * grad
-
-def select_activation(name):
-    fn = {
-        'swish': SwishFn.apply,
-        'mish': MishFn.apply,
-        'elu': F.elu,
-        'relu': F.relu,
-        'gelu': F.gelu,
-        'leaky_relu': partial(F.leaky_relu, negative_slope = 0.1)
-    }[name.lower()]
-    return lambda: Activation(fn)
-
-# evonorm
-
-def group_std(x, groups = 32, eps = 1e-5):  
-    shape = x.shape 
-    x = rearrange(x, 'b (g c) h w -> b g c h w', g = groups)    
-    var = torch.var(x, dim = (2, 3, 4), keepdim = True).expand_as(x)    
-    return torch.reshape(torch.sqrt(var + eps), shape)  
-
-class EvoNorm2d(nn.Module): 
-    def __init__(   
-        self,   
-        input,  
-        groups = 32,
-        eps = 1e-5, 
-    ):  
-        super().__init__()  
-        self.eps = eps  
-        self.groups = groups
-        self.gamma = nn.Parameter(torch.ones(1, input, 1, 1))   
-        self.beta = nn.Parameter(torch.zeros(1, input, 1, 1))   
-        self.v = nn.Parameter(torch.ones(1, input, 1, 1))   
-
-    def forward(self, x):   
-        num = x * torch.sigmoid(self.v * x) 
-        return num / group_std(x, groups = self.groups, eps = self.eps) * self.gamma + self.beta
-
 # helper classes
 
 class NanException(Exception):
@@ -252,6 +164,9 @@ def resize_to_minimum_size(min_size, image):
         return torchvision.transforms.functional.resize(image, min_size)
     return image
 
+def normalize_image(img):
+    return img * 2 - 1
+
 class ImageDataset(Dataset):
     def __init__(self, folder, image_size, transparent = False, aug_prob = 0.):
         super().__init__()
@@ -269,7 +184,8 @@ class ImageDataset(Dataset):
             transforms.Resize(image_size),
             RandomApply(aug_prob, transforms.RandomResizedCrop(image_size, scale=(0.5, 1.0), ratio=(0.98, 1.02)), transforms.CenterCrop(image_size)),
             transforms.ToTensor(),
-            transforms.Lambda(expand_greyscale(transparent))
+            transforms.Lambda(expand_greyscale(transparent)),
+            transforms.Lambda(normalize_image)
         ])
 
     def __len__(self):
@@ -304,9 +220,10 @@ class AugWrapper(nn.Module):
 
 # modifiable global variables
 
-activation_fn = select_activation('leaky_relu')
 norm_class = nn.BatchNorm2d
-conv2d = nn.Conv2d
+
+def upsample():
+    return nn.Upsample(scale_factor = 2)
 
 # classes
 
@@ -321,7 +238,7 @@ class SLE(nn.Module):
         self.net = nn.Sequential(
             nn.AdaptiveAvgPool2d((4, 4)),
             nn.Conv2d(chan_in, chan_in, 4),
-            activation_fn(),
+            nn.LeakyReLU(0.1),
             nn.Conv2d(chan_in, chan_out, 1),
             nn.Sigmoid()
         )
@@ -389,8 +306,8 @@ class Generator(nn.Module):
 
             layer = nn.ModuleList([
                 nn.Sequential(
-                    nn.Upsample(scale_factor = 2),
-                    conv2d(chan_in, chan_out * 2, 3, padding = 1),
+                    upsample(),
+                    nn.Conv2d(chan_in, chan_out * 2, 3, padding = 1),
                     norm_class(chan_out * 2),
                     nn.GLU(dim = 1)
                 ),
@@ -421,7 +338,7 @@ class Generator(nn.Module):
             if res in residuals:
                 x = x * residuals[res]
 
-        return self.out_conv(x)
+        return self.out_conv(x).tanh()
 
 class SimpleDecoder(nn.Module):
     def __init__(
@@ -441,8 +358,8 @@ class SimpleDecoder(nn.Module):
             last_layer = ind == (num_upsamples - 1)
             chan_out = chans if not last_layer else final_chan * 2
             layer = nn.Sequential(
-                nn.Upsample(scale_factor = 2),
-                conv2d(chans, chan_out, 3, padding = 1),
+                upsample(),
+                nn.Conv2d(chans, chan_out, 3, padding = 1),
                 nn.GLU(dim = 1)
             )
             self.layers.append(layer)
@@ -491,8 +408,8 @@ class Discriminator(nn.Module):
             chan_out = features[0][-1] if last_layer else init_channel
 
             self.non_residual_layers.append(nn.Sequential(
-                conv2d(init_channel, chan_out, 4, stride = 2, padding = 1),
-                activation_fn()
+                nn.Conv2d(init_channel, chan_out, 4, stride = 2, padding = 1),
+                nn.LeakyReLU(0.1)
             ))
 
         self.residual_layers = nn.ModuleList([])
@@ -508,28 +425,28 @@ class Discriminator(nn.Module):
 
             self.residual_layers.append(nn.ModuleList([
                 nn.Sequential(
-                    conv2d(chan_in, chan_out, 4, stride = 2, padding = 1),
-                    activation_fn(),
-                    conv2d(chan_out, chan_out, 3, padding = 1),
-                    activation_fn()
+                    WN(nn.Conv2d(chan_in, chan_out, 4, stride = 2, padding = 1)),
+                    nn.LeakyReLU(0.1),
+                    WN(nn.Conv2d(chan_out, chan_out, 3, padding = 1)),
+                    nn.LeakyReLU(0.1)
                 ),
                 nn.Sequential(
                     nn.AvgPool2d(2),
-                    conv2d(chan_in, chan_out, 1),
-                    activation_fn()
+                    WN(nn.Conv2d(chan_in, chan_out, 1)),
+                    nn.LeakyReLU(0.1),
                 ),
                 hamburger
             ]))
 
         last_chan = features[-1][-1]
         self.to_logits = nn.Sequential(
-            conv2d(last_chan, last_chan, 1) if disc_output_size == 5 else nn.Conv2d(last_chan, last_chan, 4, stride = 2, padding = 1),
-            activation_fn(),
+            WN(nn.Conv2d(last_chan, last_chan, 1)) if disc_output_size == 5 else nn.Conv2d(last_chan, last_chan, 4, stride = 2, padding = 1),
+            nn.LeakyReLU(0.1),
             nn.Conv2d(last_chan, 1, 4)
         )
 
         self.decoder1 = SimpleDecoder(chan_in = last_chan, chan_out = init_channel)
-        self.decoder2 = SimpleDecoder(chan_in = features[-2][-1], chan_out = init_channel)
+        self.decoder2 = SimpleDecoder(chan_in = features[-2][-1], chan_out = init_channel) if resolution >= 9 else None
 
     def forward(self, x, calc_aux_loss = False):
         orig_img = x
@@ -558,23 +475,24 @@ class Discriminator(nn.Module):
 
         recon_img_8x8 = self.decoder1(layer_8x8)
 
-        aux_loss1 = F.mse_loss(
+        aux_loss = F.mse_loss(
             recon_img_8x8,
             F.interpolate(orig_img, size = recon_img_8x8.shape[2:])
         )
 
-        select_random_quadrant = lambda rand_quadrant, img: rearrange(img, 'b c (m h) (n w) -> (m n) b c h w', m = 2, n = 2)[rand_quadrant]
-        crop_image_fn = partial(select_random_quadrant, floor(random() * 4))
-        img_part, layer_16x16_part = map(crop_image_fn, (orig_img, layer_16x16))
+        if exists(self.decoder2):
+            select_random_quadrant = lambda rand_quadrant, img: rearrange(img, 'b c (m h) (n w) -> (m n) b c h w', m = 2, n = 2)[rand_quadrant]
+            crop_image_fn = partial(select_random_quadrant, floor(random() * 4))
+            img_part, layer_16x16_part = map(crop_image_fn, (orig_img, layer_16x16))
 
-        recon_img_16x16 = self.decoder2(layer_16x16_part)
+            recon_img_16x16 = self.decoder2(layer_16x16_part)
 
-        aux_loss2 = F.mse_loss(
-            recon_img_16x16,
-            F.interpolate(img_part, size = recon_img_16x16.shape[2:])
-        )
+            aux_loss_16x16 = F.mse_loss(
+                recon_img_16x16,
+                F.interpolate(img_part, size = recon_img_16x16.shape[2:])
+            )
 
-        aux_loss = aux_loss1 + aux_loss2
+            aux_loss = aux_loss + aux_loss_16x16
 
         return out, aux_loss
 
@@ -671,7 +589,6 @@ class Trainer():
         mixed_prob = 0.9,
         gradient_accumulate_every = 1,
         hamburger_res_layers = [],
-        use_evonorm = False,
         disc_output_size = 5,
         lr = 2e-4,
         lr_mlp = 1.,
@@ -687,7 +604,6 @@ class Trainer():
         rank = 0,
         world_size = 1,
         log = False,
-        activation = 'leaky_relu',
         *args,
         **kwargs
     ):
@@ -722,8 +638,6 @@ class Trainer():
 
         self.gradient_accumulate_every = gradient_accumulate_every
 
-        self.activation = activation
-        self.use_evonorm = use_evonorm
         self.hamburger_res_layers = hamburger_res_layers
         self.disc_output_size = disc_output_size
 
@@ -760,12 +674,8 @@ class Trainer():
 
         # set some global variables before instantiating GAN
 
-        global activation_fn
         global norm_class
-
-        activation_fn = select_activation(self.activation)
-        norm_class = nn.BatchNorm2d if not self.use_evonorm else lambda chans: EvoNorm2d(chans, groups = 32) if chans >= 64 else nn.Identity()
-        norm_class = nn.SyncBatchNorm if not self.use_evonorm and self.syncbatchnorm else norm_class
+        norm_class = nn.SyncBatchNorm if self.syncbatchnorm else nn.BatchNorm2d
 
         # handle bugs when
         # switching from multi-gpu back to single gpu
@@ -806,10 +716,7 @@ class Trainer():
         config = self.config() if not self.config_path.exists() else json.loads(self.config_path.read_text())
         self.image_size = config['image_size']
         self.transparent = config['transparent']
-        self.use_evonorm = config['use_evonorm']
         self.hamburger_res_layers = config['hamburger_res_layers']
-        self.use_evonorm = config['use_evonorm']
-        self.activation = config['activation']
         self.syncbatchnorm = config['syncbatchnorm']
         self.fmap_max = config.pop('fmap_max', 512)
         del self.GAN
@@ -819,10 +726,8 @@ class Trainer():
         return {
             'image_size': self.image_size,
             'transparent': self.transparent,
-            'use_evonorm': self.use_evonorm,
             'syncbatchnorm': self.syncbatchnorm,
             'hamburger_res_layers': self.hamburger_res_layers,
-            'activation': self.activation
         }
 
     def set_data_src(self, folder):
@@ -854,8 +759,6 @@ class Trainer():
         D = self.GAN.D if not self.is_ddp else self.D_ddp
         D_aug = self.GAN.D_aug if not self.is_ddp else self.D_aug_ddp
 
-        apply_gradient_penalty = self.steps % 4 == 0
-
         # train discriminator
 
         self.GAN.D_opt.zero_grad()
@@ -880,11 +783,6 @@ class Trainer():
             self.last_recon_loss = aux_loss.clone().detach().item()
             disc_loss = disc_loss + aux_loss
 
-            if apply_gradient_penalty:
-                gp = gradient_penalty(image_batch, (real_output,))
-                self.last_gp_loss = gp.clone().detach().item()
-                disc_loss = disc_loss + gp
-
             disc_loss = disc_loss / self.gradient_accumulate_every
             disc_loss.register_hook(raise_if_nan)
             disc_loss.backward()
@@ -1016,7 +914,7 @@ class Trainer():
     @torch.no_grad()
     def generate_truncated(self, G, style, trunc_psi = 0.75, num_image_tiles = 8):
         generated_images = evaluate_in_chunks(self.batch_size, G, style)
-        return generated_images.clamp_(0., 1.)
+        return (generated_images + 1) * 0.5
 
     @torch.no_grad()
     def generate_interpolation(self, num = 0, num_image_tiles = 8, trunc = 1.0, num_steps = 100, save_frames = False):
@@ -1059,7 +957,6 @@ class Trainer():
         data = [
             ('G', self.g_loss),
             ('D', self.d_loss),
-            ('GP', self.last_gp_loss),
             ('SS', self.last_recon_loss),
             ('FID', self.last_fid)
         ]
